const debug = require("debug")("hoxtoncli:api:data");
const fs = require("fs");
const path = require("path");
const Router = require("koa-router");
const globp = require("../../lib/globp");

const HOXTON_DATA_DIR = ".hoxton-data";
const getDataFilePath = (file) =>
    path.join(process.cwd(), HOXTON_DATA_DIR, file);

/**
 * List all of the files in the local hoxton data directory
 *
 * @param {Koa.Context} ctx
 */
const listHoxtonData = async (ctx) => {
    try {
        const cwd = process.cwd();
        const files = await globp(getDataFilePath("**"));

        ctx.body = {
            status: "success",
            data: {
                files: files.map((filePath) => ({
                    path: path.relative(cwd, filePath),
                })),
            },
        };
    } catch (err) {
        debug("Error", err);
        ctx.status = 500;
    }
};

/**
 * Save the JSON data passed in the request to the specified file in the hoxton data directory
 *
 * @param {Koa.Context} ctx
 */
const saveHoxtonData = async (ctx) => {
    try {
        const input = ctx.request.body;
        if (!input || !ctx.request.is("application/json")) {
            ctx.status = 400;
            return;
        }

        const dataFilePath = getDataFilePath(ctx.params.filePath);
        await fs.promises.mkdir(path.dirname(dataFilePath), {
            recursive: true,
        });
        await fs.promises.writeFile(dataFilePath, JSON.stringify(input));

        ctx.body = { status: "success" };
    } catch (err) {
        debug("Error", err);
        ctx.status = 500;
    }
};

/**
 * Get the contents of the specified file in the hoxton data directory
 *
 * @param {Koa.Context} ctx
 */
const getHoxtonData = async (ctx) => {
    try {
        const dataFilePath = getDataFilePath(ctx.params.filePath);
        const fileContents = await fs.promises.readFile(dataFilePath);
        ctx.body = { status: "success", data: JSON.parse(fileContents) };
    } catch (err) {
        debug("Error", err);
        ctx.status = 404;
    }
};

// Create the routes for the API
const router = new Router();
router.get("/", listHoxtonData);
router.get("/:filePath(.+)", getHoxtonData);
router.post("/:filePath(.+)", saveHoxtonData);

module.exports = {
    router,
    HOXTON_DATA_DIR,
};
