const debug = require("debug")("hoxtoncli:inject:livereload");
const escapeStringRegexp = require("escape-string-regexp");
const fs = require("fs");
const livereload = require("livereload");
const { insertAt } = require("posthtml-insert-at");
const { HOXTON_DATA_DIR } = require("../api/data");

/**
 * Get the contents of the certificate and private keys files
 *
 * Reads files in Sync mode as the server cannot be loaded asynchronously due
 * to the way the middlewares are loaded by LWS
 *
 * @param {Object} config the `lws` config object
 */
const getCertificate = (config) => {
    debug(`Certificate file: ${config.cert}`);
    debug(`Private key file: ${config.key}`);
    return {
        cert: fs.readFileSync(config.cert),
        key: fs.readFileSync(config.key),
    };
};

/**
 * Starts a live reload server
 *
 * @param {Object} config the `lws` config object
 */
const startLiveReloadServer = (config) => {
    // TODO what if port in use?

    const port = config.port + 1;
    const server = livereload.createServer({
        port,
        https: getCertificate(config),
        extraExts: ["json"],
        exclusions: [
            new RegExp(escapeStringRegexp(HOXTON_DATA_DIR)),
            /package\.json/,
        ],
    });
    debug(`Live reload server running on port: ${port}`);

    const cwd = process.cwd();
    server.watch(cwd);
    debug(`Live reload server watching: ${cwd}`);

    return { port };
};

/**
 * Returns the JS code needed to handle live reload events
 *
 * @param {Number} port the port that the live reload server is running on
 */
const getLiveReloadSnippet = (port) => {
    return [
        "<!-- livereload -->",
        "<script>",
        "document.write('<script src=\"' + location.protocol + '//'",
        " + (location.host || 'localhost').split(':')[0]",
        ` + ':${port}/livereload.js?snipver=1"><\\/script>')`,
        "</script>",
        "",
    ].join("\n");
};

/**
 * Injects the JS needed for the page to respond to live reload events
 *
 * @param {PostHTML.PostHTML} posthtmlInstance the instance of posthtml to work with
 * @param {Number} port the port that the live reload server is running on
 * @returns {PostHTML.PostHTML}
 */
const injectLiveReload = (posthtmlInstance, port) => {
    const injectText = getLiveReloadSnippet(port);
    posthtmlInstance.use(
        insertAt({
            selector: "head",
            append: injectText,
        })
    );

    return posthtmlInstance;
};

module.exports = {
    injectLiveReload,
    startLiveReloadServer,
};
