const fs = require("fs");
const path = require("path");
const { insertAt } = require("posthtml-insert-at");
const removeTags = require("posthtml-remove-tags");

/**
 * Get the contents of a manifest file if it exists
 *
 * @param {Koa.Context} ctx the request context
 * @returns {Object|null}
 */
const getExternalManifest = async (ctx) => {
    // Get the manifest external manifest file - it has to be next to the index.html
    const filePath = path.join(
        process.cwd(),
        decodeURIComponent(ctx.path.replace("index.html", "hoxton.json"))
    );
    try {
        // Check if the file exists / is readable
        await fs.promises.access(filePath);
        // Read the file and return its contents
        const contents = await fs.promises.readFile(filePath);
        return contents.toString("utf8").trim();
    } catch (err) {
        // File does not exist
        return null;
    }
};

/**
 * Adds hoxton manifest into the index.html response
 *
 * @param {PostHTML.PostHTML} posthtmlInstance the instance of posthtml to work with
 * @param {String} manifest the hoxton.json manifest to inject
 * @returns {PostHTML.PostHTML}
 */
const injectExternalManifest = (posthtmlInstance, manifest) => {
    // Remove the manifest if it exists already
    posthtmlInstance.use(removeTags({ tags: ["hoxton"] }));
    // Inject the manifest into the HTML
    posthtmlInstance.use(
        insertAt({
            selector: "head",
            append: `<hoxton style="display:none" data='${encodeURIComponent(
                encodeURIComponent(manifest)
            )}'></hoxton>\n`,
        })
    );
    return posthtmlInstance;
};

module.exports = {
    getExternalManifest,
    injectExternalManifest,
};
